/*
 * Copyright (c) 2020 Nordic Semiconductor ASA
 *
 * SPDX-License-Identifier: LicenseRef-Nordic-5-Clause
 */

#include <zephyr/kernel.h>
#include <zephyr/logging/log.h>
#include "Watchdog.h"
#include <ram_pwrdn.h>
#include <zephyr/logging/log_ctrl.h>
#include "reset_reason.h"
#include <nrfx_gpiote.h>
#include <zephyr/sys/reboot.h>
static void hardware_init(void);

LOG_MODULE_REGISTER(MAIN, 4);
#define DK_LED_ONE NRF_GPIO_PIN_MAP(0,28) 

//#define DK_LED_ONE NRF_GPIO_PIN_MAP(0,1) //alt board led

void set_dk_led(void);

void set_dk_led(void)
{
    // Initialize the GPIOTE module
    nrfx_err_t err =nrfx_gpiote_init(0);
	 if (err != NRFX_SUCCESS) 
        LOG_ERR("Failed nrfx init %X",err); 
    nrfx_gpiote_out_config_t pin_config = NRFX_GPIOTE_CONFIG_OUT_SIMPLE(true);
    err = nrfx_gpiote_out_init(DK_LED_ONE, &pin_config);
    if (err != NRFX_SUCCESS) 
    	LOG_DBG("failed nrfx gpiote out init %X",err); 
    // Set the pin low
    nrfx_gpiote_out_clear(DK_LED_ONE); //led active
     LOG_DBG("pin clear"); 
}

void trigger_reset (void)
{
	  // power up unused ram for mcuboot
    if (IS_ENABLED(CONFIG_RAM_POWER_DOWN_LIBRARY))
    {
        power_up_unused_ram();
    }
   // resume_external_flash();
   sys_reboot(SYS_REBOOT_COLD);
   // NVIC_SystemReset();//<< Replaced sys_reboot with NVIC_SystemReset() to implement the workaround for nRF5340 eratta

}


void main(void)
{
	LOG_INF("Starting Demo Firmware");

	if (IS_ENABLED(CONFIG_RAM_POWER_DOWN_LIBRARY)) 
	{
		power_down_unused_ram();
	}
	check_reset_reason();
	int err;
	err = watchdog_init();
	if(err <0 )
	{
		LOG_INF("Watchdog Initialisation Error %d",err);
		return;
	}

	err = watchdog_start();
	if(err < 0 )
	{
		LOG_INF("Could not start Watchdog");
		return;
	}
	LOG_DBG("Resetting the device!");
	set_dk_led();
    k_sleep(K_MSEC(2000)); //For pushing out logs

	while(true)
	{
		err = watchdog_feed();
		trigger_reset();
		k_sleep(K_SECONDS(25));	
	}
}
		

